@@ -15,9 +15,11 @@ from typing import (
     TYPE_CHECKING,
     Any,
     Callable,
+    ForwardRef,
     Generic,
     Iterable,
     Iterator,
+    List,
     Literal,
     Sequence,
     Tuple,
@@ -29,7 +31,7 @@ from typing import (
 )
 from weakref import ref
 
-from typing_extensions import get_args, get_origin
+from typing_extensions import Annotated, get_args, get_origin
 
 from magicgui._type_resolution import resolve_single_type
 from magicgui._util import merge_super_sigs, safe_issubclass
@@ -52,7 +54,12 @@ from magicgui.widgets.bases import (
 from magicgui.widgets.bases._mixins import _OrientationMixin, _ReadOnlyMixin
 
 if TYPE_CHECKING:
+    from typing_extensions import Unpack
+
     from magicgui.widgets import protocols
+    from magicgui.widgets.bases._container_widget import ContainerKwargs
+    from magicgui.widgets.bases._widget import WidgetKwargs
+
 
 WidgetVar = TypeVar("WidgetVar", bound=Widget)
 WidgetTypeVar = TypeVar("WidgetTypeVar", bound=Type[Widget])
@@ -281,32 +288,13 @@ class LogSlider(TransformedRangedWidget):
         max: float = 100,
         base: float = math.e,
         tracking: bool = True,
-        **kwargs: Any,
+        **kwargs: Unpack[WidgetKwargs],
     ):
-        # sourcery skip: avoid-builtin-shadow
-        for key in ("maximum", "minimum"):
-            if key in kwargs:
-                import warnings
-
-                warnings.warn(
-                    f"The {key!r} keyword arguments has been changed to {key[:3]!r}. "
-                    "In the future this will raise an exception\n",
-                    FutureWarning,
-                    stacklevel=2,
-                )
-                if key == "maximum":
-                    max = kwargs.pop(key)  # noqa: A001
-                else:
-                    min = kwargs.pop(key)  # noqa: A001
         self._base = base
         app = use_app()
         assert app.native
-        super().__init__(
-            min=min,
-            max=max,
-            widget_type=app.get_obj("Slider"),
-            **kwargs,
-        )
+        kwargs["widget_type"] = app.get_obj("Slider")
+        super().__init__(min=min, max=max, **kwargs)
         self.tracking = tracking
 
     @property
@@ -369,7 +357,10 @@ class RadioButtons(CategoricalWidget, _OrientationMixin):  # type: ignore
     """An exclusive group of radio buttons, providing a choice from multiple choices."""
 
     def __init__(
-        self, choices: ChoicesType = (), orientation: str = "vertical", **kwargs: Any
+        self,
+        choices: ChoicesType = (),
+        orientation: str = "vertical",
+        **kwargs: Unpack[WidgetKwargs],
     ) -> None:
         app = use_app()
         assert app.native
@@ -382,8 +373,9 @@ class RadioButtons(CategoricalWidget, _OrientationMixin):  # type: ignore
 class Container(ContainerWidget[WidgetVar]):
     """A Widget to contain other widgets.
 
-    Note that `Container` implements the [`typing.MutableSequence`][] interface,
-    so you can use it like a list to add and remove widgets.
+    Note that `Container` implements the
+    [`typing.MutableSequence`][typing.MutableSequence]
+    interface, so you can use it like a list to add and remove widgets.
     """
 
 
@@ -419,9 +411,10 @@ class FileEdit(Container):
         mode: FileDialogMode = FileDialogMode.EXISTING_FILE,
         filter: str | None = None,
         nullable: bool = False,
-        **kwargs: Any,
+        **kwargs: Unpack[ContainerKwargs],
     ) -> None:
-        value = kwargs.pop("value", None)
+        # use empty string as a null value
+        value = kwargs.pop("value", None)  # type: ignore [typeddict-item]
         if value is None:
             value = ""
         self.line_edit = LineEdit(value=value)
@@ -530,9 +523,9 @@ class RangeEdit(Container[SpinBox]):
         step: int = 1,
         min: int | tuple[int, int, int] | None = None,
         max: int | tuple[int, int, int] | None = None,
-        **kwargs: Any,
+        **kwargs: Unpack[ContainerKwargs],
     ) -> None:
-        value = kwargs.pop("value", None)
+        value = kwargs.pop("value", None)  # type: ignore [typeddict-item]
         if value is not None and value is not Undefined:
             if not all(hasattr(value, x) for x in ("start", "stop", "step")):
                 raise TypeError(f"Invalid value type for {type(self)}: {type(value)}")
@@ -545,7 +538,7 @@ class RangeEdit(Container[SpinBox]):
         kwargs["widgets"] = [self.start, self.stop, self.step]
         kwargs.setdefault("layout", "horizontal")
         kwargs.setdefault("labels", True)
-        kwargs.pop("nullable", None)
+        kwargs.pop("nullable", None)  # type: ignore [typeddict-item]
         super().__init__(**kwargs)
 
     @classmethod
@@ -628,13 +621,13 @@ class ListEdit(Container[ValueWidget[_V]]):
         All additional keyword arguments are passed to `Container` constructor.
     """
 
-    def __init__(
+    def __init__(  # type: ignore [misc]  # overlap between names
         self,
         value: Iterable[_V] | _Undefined = Undefined,
         layout: str = "horizontal",
         nullable: bool = False,
         options: dict | None = None,
-        **kwargs: Any,
+        **kwargs: Unpack[ContainerKwargs],
     ) -> None:
         self._args_type: type | None = None
         self._nullable = nullable
@@ -690,17 +683,27 @@ class ListEdit(Container[ValueWidget[_V]]):
             self._args_type = None
             return
 
-        value = resolve_single_type(value)
+        value_resolved = resolve_single_type(value)
+        if isinstance(value, (str, ForwardRef)):
+            value = value_resolved
+        # unwrap annotated (options are not needed to normalize `annotation`)
+        while get_origin(value) is Annotated:
+            value = get_args(value)[0]
         arg: type | None = None
 
-        if value and value is not inspect.Parameter.empty:
-            orig = get_origin(value) or value
+        if value_resolved and value_resolved is not inspect.Parameter.empty:
+            orig = get_origin(value_resolved) or value_resolved
             if not (safe_issubclass(orig, list) or isinstance(orig, list)):
                 raise TypeError(
                     f"cannot set annotation {value} to {type(self).__name__}."
                 )
             args = get_args(value)
             arg = args[0] if len(args) > 0 else None
+            args_resolved = get_args(value_resolved)
+            if len(args_resolved) > 0:
+                value = List[args_resolved[0]]  # type: ignore
+            else:
+                value = list
 
         self._annotation = value
         self._args_type = arg
@@ -875,14 +878,14 @@ class TupleEdit(Container[ValueWidget]):
         self,
         value: Iterable[_V] | _Undefined = Undefined,
         *,
-        layout: str = "horizontal",
         nullable: bool = False,
         options: dict | None = None,
-        **kwargs: Any,
+        **container_kwargs: Unpack[ContainerKwargs[ValueWidget]],
     ) -> None:
         self._nullable = nullable
         self._args_types: tuple[type, ...] | None = None
-        super().__init__(layout=layout, labels=False, **kwargs)
+        container_kwargs["labels"] = False
+        super().__init__(**container_kwargs)
         self._child_options = options or {}
         self.margins = (0, 0, 0, 0)
 
@@ -929,17 +932,23 @@ class TupleEdit(Container[ValueWidget]):
             self._args_types = None
             return
 
-        value = resolve_single_type(value)
+        value_resolved = resolve_single_type(value)
+        if isinstance(value, (str, ForwardRef)):
+            value = value_resolved
+        # unwrap annotated (options are not needed to normalize `annotation`)
+        while get_origin(value) is Annotated:
+            value = get_args(value)[0]
         args: tuple[type, ...] | None = None
 
-        if value and value is not inspect.Parameter.empty:
-            orig = get_origin(value)
+        if value_resolved and value_resolved is not inspect.Parameter.empty:
+            orig = get_origin(value_resolved)
             if not (safe_issubclass(orig, tuple) or isinstance(orig, tuple)):
                 raise TypeError(
                     f"cannot set annotation {value} to {type(self).__name__}."
                 )
             args = get_args(value)
-            value = Tuple[args]
+            args_resolved = get_args(value_resolved)
+            value = Tuple[args_resolved]
 
         self._annotation = value
         self._args_types = args
@@ -963,12 +972,12 @@ class TupleEdit(Container[ValueWidget]):
 class _LabeledWidget(Container):
     """Simple container that wraps a widget and provides a label."""
 
-    def __init__(
+    def __init__(  # type: ignore [misc]  # overlap between argument names
         self,
         widget: Widget,
         label: str | None = None,
         position: str = "left",
-        **kwargs: Any,
+        **kwargs: Unpack[ContainerKwargs],
     ) -> None:
         kwargs["layout"] = "horizontal" if position in {"left", "right"} else "vertical"
         self._inner_widget = widget
