@@ -31,9 +31,12 @@ from magicgui.widgets.bases._value_widget import ValueWidget
 if TYPE_CHECKING:
     import numpy
     import pandas
-    from typing_extensions import TypeGuard
+    from typing_extensions import TypeGuard, Unpack
 
     from magicgui.widgets.protocols import TableWidgetProtocol
+
+    from .bases._widget import WidgetKwargs
+
 TblKey = Any
 _KT = TypeVar("_KT")  # Key type
 _KT_co = TypeVar("_KT_co", covariant=True)  # Key type covariant containers.
@@ -160,8 +163,8 @@ class Table(ValueWidget, _ReadOnlyMixin, MutableMapping[TblKey, list]):
         ``tuple(range(len(data[0])))``.  Values provided here override any implied in
         ``value``.
     **kwargs
-        Additional kwargs will be passed to the [magicgui.widgets.Widget][]
-        constructor.
+        Additional kwargs will be passed to the
+        [magicgui.widgets.Widget][magicgui.widgets.Widget] constructor.
 
     Attributes
     ----------
@@ -205,7 +208,7 @@ class Table(ValueWidget, _ReadOnlyMixin, MutableMapping[TblKey, list]):
         Emitted whenever a cell in the table changes. The value will have a
         dict of information regarding the cell that changed:
         {'data': x, 'row': int, 'column': int, 'column_header': str, 'row_header': str}
-        CURRENTLY: only emitted on changes in the GUI. not programattic changes.
+        CURRENTLY: only emitted on changes in the GUI. not programmatic changes.
     """
 
     _widget: TableWidgetProtocol
@@ -227,9 +230,10 @@ class Table(ValueWidget, _ReadOnlyMixin, MutableMapping[TblKey, list]):
         *,
         index: Collection | None = None,
         columns: Collection | None = None,
-        **kwargs: Any,
+        **kwargs: Unpack[WidgetKwargs],
     ) -> None:
-        super().__init__(widget_type=use_app().get_obj("Table"), **kwargs)
+        kwargs["widget_type"] = use_app().get_obj("Table")
+        super().__init__(**kwargs)
         self._data = DataView(self)
         data, _index, _columns = normalize_table_data(value)
         self.value = {
