@@ -1,8 +1,9 @@
 """Protocols (interfaces) for backends to implement.
 
-Each class in this module is a [typing.Protocol][] that specifies a set of
-:func:`~abc.abstractmethod` that a backend widget must implement to be compatible with
-the magicgui API.  All magicgui-specific abstract methods are prefaced with ``_mgui_*``.
+Each class in this module is a [typing.Protocol][typing.Protocol] that specifies
+a set of :func:`~abc.abstractmethod` that a backend widget must implement
+to be compatible with the magicgui API.
+All magicgui-specific abstract methods are prefaced with ``_mgui_*``.
 
 For an example backend implementation, see ``magicgui.backends._qtpy.widgets``
 """
@@ -437,7 +438,21 @@ class SupportsText(Protocol):
 
 
 @runtime_checkable
-class ButtonWidgetProtocol(ValueWidgetProtocol, SupportsText, Protocol):
+class SupportsIcon(Protocol):
+    """Widget that can be reoriented."""
+
+    @abstractmethod
+    def _mgui_set_icon(self, value: str | None, color: str | None) -> None:
+        """Set icon.
+
+        Value is an "prefix:name" from iconify: https://icon-sets.iconify.design
+        Color is any valid CSS color string.
+        Set value to `None` or an empty string to remove icon.
+        """
+
+
+@runtime_checkable
+class ButtonWidgetProtocol(ValueWidgetProtocol, SupportsText, SupportsIcon, Protocol):
     """The "value" in a ButtonWidget is the current (checked) state."""
 
 
