@@ -6,6 +6,7 @@ from typing import Optional, Tuple
 from unittest.mock import MagicMock, patch
 
 import pytest
+from typing_extensions import Annotated
 
 from magicgui import magicgui, types, use_app, widgets
 from magicgui.widgets import Container, request_values
@@ -109,6 +110,17 @@ def test_create_widget_annotation(annotation, expected_type):
     wdg.close()
 
 
+def test_create_widget_annotation_overwritte_parrams():
+    wdg1 = widgets.create_widget(annotation=widgets.ProgressBar)
+    assert isinstance(wdg1, widgets.ProgressBar)
+    assert wdg1.visible
+    wdg2 = widgets.create_widget(
+        annotation=Annotated[widgets.ProgressBar, {"visible": False}]
+    )
+    assert isinstance(wdg2, widgets.ProgressBar)
+    assert not wdg2.visible
+
+
 # fmt: off
 class MyBadWidget:
     """INCOMPLETE widget implementation and will error."""
@@ -197,7 +209,7 @@ def test_basic_widget_attributes():
 
     assert widget.parent is None
     container.append(widget)
-    assert widget.parent is container.native
+    assert widget.parent is container
     widget.parent = None
     assert widget.parent is None
     assert widget.label == "my name"
@@ -808,6 +820,17 @@ def test_pushbutton_click_signal():
     mock2.assert_called_once()
 
 
+def test_pushbutton_icon(backend: str):
+    use_app(backend)
+    btn = widgets.PushButton(icon="mdi:folder")
+    btn.set_icon("play", "red")
+    btn.set_icon(None)
+
+    if backend == "qt":
+        with pytest.warns(UserWarning, match="Could not set iconify icon"):
+            btn.set_icon("bad:key")
+
+
 def test_list_edit():
     """Test ListEdit."""
     from typing import List
@@ -906,6 +929,15 @@ def test_list_edit():
     assert type(f4.x[0]) is widgets.SpinBox
     assert f4.x.value == [0]
 
+    @magicgui
+    def f5(x: List[Annotated[int, {"max": 3}]]):
+        pass
+
+    assert type(f5.x) is widgets.ListEdit
+    assert f5.x.annotation == List[int]
+    f5.x.btn_plus.changed()
+    assert f5.x[0].max == 3
+
 
 def test_tuple_edit():
     """Test TupleEdit."""
@@ -946,6 +978,14 @@ def test_tuple_edit():
     assert f2.x.annotation == Tuple[int, str]
     assert f2.x.value == (0, "")
 
+    @magicgui
+    def f3(x: Tuple[Annotated[int, {"max": 3}], str]):
+        pass
+
+    assert type(f3.x) is widgets.TupleEdit
+    assert f2.x.annotation == Tuple[int, str]
+    assert f3.x[0].max == 3
+
 
 def test_request_values(monkeypatch):
     from unittest.mock import Mock
